// SPDX-FileCopyrightText: 2020 CERN (home.cern)
//
// SPDX-License-Identifier: CC0-1.0

`default_nettype none
`timescale 1ps/1ps

module tb_iserdes;

   logic reset;
   logic clk_sys;

   logic adc0_dco;
   logic adc0_fr;
   logic ext_trig;
   logic adc_data_dir;

   logic[3:0] adc0_dat_odd;
   logic[3:0] adc0_dat_even;
   logic signed [13:0] adc0_data;
   wire [63:0] adc_data;

   logic serdes_calib = 1'b0;
   wire serdes_locked;
   wire serdes_synced;

  parameter tser = 1.25ps;

  initial begin
    reset = 1;
    clk_sys = 0;
    adc0_dco = 0;
    adc0_fr = 1'b0;
    ext_trig = 1'b0;
    adc_data_dir = 1'b0;
    adc0_dat_odd  = 4'h0;
    adc0_dat_even = 4'h0;
  end

   // 400Mhz
   always #1.25ns adc0_dco <= ~adc0_dco;

   // 100Mhz
   always #5ns clk_sys <= ~clk_sys;

   initial begin
      repeat(10) @(posedge clk_sys);
      reset = 0;
   end
	 
	 ltc2174_2l16b_us_receiver ltc2174_inst (
    // ADC data clock
    .adc_dco_p_i(~adc0_dco),
    .adc_dco_n_i(adc0_dco),
    // ADC frame start
    .adc_fr_p_i(~adc0_fr),
    .adc_fr_n_i(adc0_fr),
    // ADC serial data in (odd bits)
    .adc_outa_p_i(adc0_dat_odd),
    .adc_outa_n_i(~adc0_dat_odd),
    // ADC serial data in (even bits)
    .adc_outb_p_i(adc0_dat_even),
    .adc_outb_n_i(~adc0_dat_even),
    // Enable serdes calibration (start with an initial calibration)
    .serdes_calib_i(serdes_calib),
    // SERDES lock status flag
    .serdes_locked_o(serdes_locked),
    // Indication that SERDES is ok and locked to
    // frame start pattern
    .serdes_synced_o(serdes_synced),
    // ADC parallel data out
    //  (15:0)  = CH1, (31:16) = CH2, (47:32) = CH3, (63:48) = CH4
    //  The two LSBs of each channel are always '0'
    .adc_data_o(adc_data),
    // ADC divided clock, for FPGA logic
    .adc_clk_o(),
    // AXI stream 64-bit data
    .m_axis_tvalid_o(),
    .m_axis_tdata_o(),
    .m_axis_tstrb_o(),
    .m_axis_tready_i(1'b1)
  );

   int adc_div = 0;

  always@(negedge adc0_dco) begin
    #625ps;
    if (adc_div == 1) begin
      adc0_fr <= ~adc0_fr;
      adc_div <= 0;
    end else begin
      adc_div <= adc_div + 1;
    end
  end

  assign adc0_data = 14'b00000001010101;

  //  Generate a triangular waveform on all channels.
  always @(posedge adc0_fr)
  begin
    /*if ((adc0_data > 400) || (adc0_data < -400)) begin
      adc_data_dir = ~adc_data_dir;
    end
	  //adc0_data <= ~adc0_data;
    if (adc_data_dir == 0) begin
      adc0_data = adc0_data + 1;
    end else begin
      adc0_data = adc0_data - 1;
    end*/
  	//#1;
    adc0_dat_odd  = {4{adc0_data[13]}};
    adc0_dat_even = {4{adc0_data[12]}};
    #1250ps;
    adc0_dat_odd  = {4{adc0_data[11]}};
    adc0_dat_even = {4{adc0_data[10]}};
    #1250ps;
    adc0_dat_odd  = {4{adc0_data[9]}};
    adc0_dat_even = {4{adc0_data[8]}};
    #1250ps;
    adc0_dat_odd  = {4{adc0_data[7]}};
    adc0_dat_even = {4{adc0_data[6]}};
    #1250ps;
    adc0_dat_odd  = {4{adc0_data[5]}};
    adc0_dat_even = {4{adc0_data[4]}};
    #1250ps;
    adc0_dat_odd  = {4{adc0_data[3]}};
    adc0_dat_even = {4{adc0_data[2]}};
    #1250ps;
    adc0_dat_odd  = {4{adc0_data[1]}};
    adc0_dat_even = {4{adc0_data[0]}};
	  #1250ps;
    adc0_dat_odd  = 4'b0;
    adc0_dat_even = 4'b0;
  end

endmodule // main
`default_nettype wire