-------------------------------------------------------------------------------
-- SPDX-License-Identifier: CERN-OHL-W-2.0+
-- Title      : FMC ADC 100Ms/s SPEXI7U top-level
-- Project    : FMC ADC 100M 14B 4CHA gateware
-- URL        : http://www.ohwr.org/projects/fmc-adc-100m14b4cha-gw
-------------------------------------------------------------------------------
-- File       : spexi7u_ref_fmc_adc_100Ms.vhd
-- Company    : CERN (BE-CEM-EDL)
-- Created    : 2023-10-01
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Top entity of FMC ADC 100Ms/s design for SPEXI7U carrier
-- See also: http://www.ohwr.org/projects/spexi7u
-------------------------------------------------------------------------------
-- Copyright (c)  CERN (BE-CEM-EDL)
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

library UNISIM;
use UNISIM.vcomponents.all;

library work;

use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.fmc_adc_mezzanine_pkg.all;

entity spexi7u_ref_fmc_adc_100Ms is
  generic(
    g_SIMULATION         : integer := 0;
    g_MULTISHOT_RAM_SIZE : natural := 2048);
  port
    (
      -- system signals
      pl_sys_clk_p_i  : in std_logic;
      pl_sys_clk_n_i  : in std_logic;
      pl_refddr_clk_n  : in std_logic;
      pl_refddr_clk_p  : in std_logic;

      pl_resetn_i  : in std_logic;

      -- misc
      led_o  : out std_logic_vector(5 downto 0);
      pmbus_alertn_i  : in std_logic;
      pl_uart_txd_i  : in std_logic;
      pl_uart_rxd_o  : in std_logic;

      -- FMC
      fmc_clk_dir  : in std_logic;
      fmc_prsnt_i  : in std_logic;
      fmc_pg_c2m_o  : in std_logic;
      fmc_pg_m2c_i  : in std_logic;

      -- DDR4
      ddr4_act_n: out std_logic;
      ddr4_adr: out std_logic_vector(16 downto 0);
      ddr4_ba: out std_logic_vector(1 downto 0);
      ddr4_bg: out std_logic_vector(0 downto 0);
      ddr4_ck_c: out std_logic_vector(0 downto 0);
      ddr4_ck_t: out std_logic_vector(0 downto 0);
      ddr4_cke: out std_logic_vector(0 downto 0);
      ddr4_cs_n: out std_logic_vector(0 downto 0);
      ddr4_dm_n: inout std_logic_vector(3 downto 0);
      ddr4_dq: inout std_logic_vector(31 downto 0);
      ddr4_dqs_c: inout std_logic_vector(3 downto 0);
      ddr4_dqs_t: inout std_logic_vector(3 downto 0);
      ddr4_odt: out std_logic_vector(0 downto 0);
      ddr4_reset_n: out std_logic;

      -- I2C
      i2c_scl_io: inout std_logic;
      i2c_sda_io: inout std_logic;

      -- PXIe
      pxie_dstara_p_i  : in std_logic;
      pxie_dstara_n_i  : in std_logic;
      pxie_dstarb_p_i  : in std_logic;
      pxie_dstarb_n_i  : in std_logic;
      pxie_dstarc_p_o: out std_logic;
      pxie_dstarc_n_o: out std_logic;
      pxie_sync100_p_i  : in std_logic;
      pxie_sync100_n_i  : in std_logic;
      pxie_clk100_p_i  : in std_logic;
      pxie_clk100_n_i  : in std_logic;
      pxie_clk10_i  : in std_logic;
      pxie_star_i  : in std_logic;
      pxie_lbr6_i  : in std_logic;
      pxie_lbl6_i  : in std_logic;
      pxie_trig_i: in std_logic_vector(7 downto 0);
      pxie_ga_i: in std_logic_vector(4 downto 0);

      -- SFP
      clk_125m_sfp_mgt_p_i  : in std_logic;
      clk_125m_sfp_mgt_n_i  : in std_logic;
      sfp_rx_n_i  : in std_logic;
      sfp_rx_p_i  : in std_logic;
      --sfp_tx_n_o  : out std_logic;
      --sfp_tx_p_o  : out std_logic;
      sfp_i2c_sda_io  : inout std_logic;
      sfp_i2c_scl_io  : inout std_logic;
      sfp_los_i  : in std_logic;
      sfp_txdis_o  : out std_logic;
      sfp_modabs_i  : in std_logic;

      -- White-Rabbit
      wr_pps_o  : out std_logic;
      wr_abscal_o  : out std_logic;
      wr_clk_helper_125m_p_i  : in std_logic;
      wr_clk_helper_125m_n_i  : in std_logic;
      clk_125m_pllref_p_i  : in std_logic;
      clk_125m_pllref_n_i  : in std_logic;
      wr_eeprom_sda_io  : inout std_logic;
      wr_eeprom_scl_io  : inout std_logic;

      plldac_sclk_o  : out std_logic;
      plldac_din_o  : out std_logic;
      pll25dac_cs_n_o  : out std_logic;
      pll20dac_cs_n_o  : out std_logic;

      ------------------------------------------
      -- FMC slot
      ------------------------------------------
      adc_ext_trigger_p_i : in std_logic;        -- External trigger
      adc_ext_trigger_n_i : in std_logic;

      adc_dco_p_i  : in std_logic;                     -- ADC data clock
      adc_dco_n_i  : in std_logic;
      adc_fr_p_i   : in std_logic;                     -- ADC frame start
      adc_fr_n_i   : in std_logic;
      adc_outa_p_i : in std_logic_vector(3 downto 0);  -- ADC serial data (odd bits)
      adc_outa_n_i : in std_logic_vector(3 downto 0);
      adc_outb_p_i : in std_logic_vector(3 downto 0);  -- ADC serial data (even bits)
      adc_outb_n_i : in std_logic_vector(3 downto 0);

      adc_spi_din_i       : in  std_logic;       -- SPI data from FMC
      adc_spi_dout_o      : out std_logic;       -- SPI data to FMC
      adc_spi_sck_o       : out std_logic;       -- SPI clock
      adc_spi_cs_adc_n_o  : out std_logic;       -- SPI ADC chip select (active low)
      adc_spi_cs_dac1_n_o : out std_logic;  -- SPI channel 1 offset DAC chip select (active low)
      adc_spi_cs_dac2_n_o : out std_logic;  -- SPI channel 2 offset DAC chip select (active low)
      adc_spi_cs_dac3_n_o : out std_logic;  -- SPI channel 3 offset DAC chip select (active low)
      adc_spi_cs_dac4_n_o : out std_logic;  -- SPI channel 4 offset DAC chip select (active low)

      adc_gpio_dac_clr_n_o : out std_logic;      -- offset DACs clear (active low)
      adc_gpio_led_acq_o   : out std_logic;      -- Mezzanine front panel power LED (PWR)
      adc_gpio_led_trig_o  : out std_logic;      -- Mezzanine front panel trigger LED (TRIG)
      adc_gpio_ssr_ch1_o   : out std_logic_vector(6 downto 0);  -- Channel 1 solid state relays control
      adc_gpio_ssr_ch2_o   : out std_logic_vector(6 downto 0);  -- Channel 2 solid state relays control
      adc_gpio_ssr_ch3_o   : out std_logic_vector(6 downto 0);  -- Channel 3 solid state relays control
      adc_gpio_ssr_ch4_o   : out std_logic_vector(6 downto 0);  -- Channel 4 solid state relays control
      adc_gpio_si570_oe_o  : out std_logic;      -- Si570 (programmable oscillator) output enable

      adc_si570_scl_b : inout std_logic;         -- I2C bus clock (Si570)
      adc_si570_sda_b : inout std_logic;         -- I2C bus data (Si570)

      adc_one_wire_b : inout std_logic  -- Mezzanine 1-wire interface (DS18B20 thermometer + unique ID)
      );

end spexi7u_ref_fmc_adc_100Ms;


architecture arch of spexi7u_ref_fmc_adc_100Ms is

  signal pl_sys_clk: std_logic;
  signal wr_clk_62m5: std_logic;
  signal sysclk_bufg: std_logic;

  signal pxie_dstara: std_logic;
  signal pxie_dstarb: std_logic;
  signal pxie_dstarc: std_logic := '0';
  signal pxie_sync100: std_logic;
  signal pxie_clk100: std_logic;

  signal i2c_scl_i: std_logic;
  signal i2c_sda_i: std_logic;
  signal i2c_ps0_scl_o: std_logic;
  signal i2c_ps0_scl_t: std_logic;
  signal i2c_ps0_sda_o: std_logic;
  signal i2c_ps0_sda_t: std_logic;

  signal ps2pl_resetn, resetn: std_logic;

  signal ps_gpio_o: std_logic_vector(94 downto 0);
  signal ps_gpio_i: std_logic_vector(94 downto 0);
  signal ext_trigger: std_logic;
  signal pxie_clk100_bufg: std_logic;

  component pxie_bd is
    port (
      ddr4_act_n : out STD_LOGIC;
      ddr4_adr : out STD_LOGIC_VECTOR(16 downto 0);
      ddr4_ba : out STD_LOGIC_VECTOR (1 downto 0);
      ddr4_bg : out STD_LOGIC_VECTOR (0 to 0);
      ddr4_ck_c : out STD_LOGIC_VECTOR(0 to 0);
      ddr4_ck_t : out STD_LOGIC_VECTOR(0 to 0);
      ddr4_cke : out STD_LOGIC_VECTOR(0 to 0);
      ddr4_cs_n : out STD_LOGIC_VECTOR(0 to 0);
      ddr4_dm_n : inout STD_LOGIC_VECTOR(3 downto 0);
      ddr4_dq : inout STD_LOGIC_VECTOR(31 downto 0);
      ddr4_dqs_c : inout STD_LOGIC_VECTOR(3 downto 0);
      ddr4_dqs_t : inout STD_LOGIC_VECTOR(3 downto 0);
      ddr4_odt : out STD_LOGIC_VECTOR(0 to 0);
      ddr4_reset_n : out STD_LOGIC;
      i2c_ps0_scl_i : in STD_LOGIC;
      i2c_ps0_scl_o : out STD_LOGIC;
      i2c_ps0_scl_t : out STD_LOGIC;
      i2c_ps0_sda_i : in STD_LOGIC;
      i2c_ps0_sda_o : out STD_LOGIC;
      i2c_ps0_sda_t : out STD_LOGIC;
      pl_refddr_clk_n : in STD_LOGIC;
      pl_refddr_clk_p : in STD_LOGIC;
      clk_i : in STD_LOGIC;
      ps2pl_resetn_o : out STD_LOGIC;
      ps_gpio_i : in STD_LOGIC_VECTOR(94 downto 0 );
      ps_gpio_o : out STD_LOGIC_VECTOR(94 downto 0 );
      resetn_i : in STD_LOGIC;
      wr_clk62m5_o : out STD_LOGIC;
      wr_eeprom_scl_io : inout STD_LOGIC;
      wr_eeprom_sda_io : inout STD_LOGIC;
      wr_led_act_o : out STD_LOGIC;
      wr_led_link_o : out STD_LOGIC;
      wr_pll20dac_csn_o : out STD_LOGIC;
      wr_pll25dac_csn_o : out STD_LOGIC;
      wr_plldac_din_o : out STD_LOGIC;
      wr_plldac_sclk_o : out STD_LOGIC;
      wr_pps_led_o : out STD_LOGIC;
      wr_pps_o : out STD_LOGIC;
      ext_trigger_p : in STD_LOGIC;
      ext_trigger_n : in STD_LOGIC;
      adc_dco_p : in STD_LOGIC;
      adc_dco_n : in STD_LOGIC;
      adc_fr_p : in STD_LOGIC;
      adc_fr_n : in STD_LOGIC;
      adc_outa_p : in STD_LOGIC_VECTOR(3 downto 0);
      adc_outa_n : in STD_LOGIC_VECTOR(3 downto 0);
      adc_outb_p : in STD_LOGIC_VECTOR(3 downto 0);
      adc_outb_n : in STD_LOGIC_VECTOR(3 downto 0);
      spi_din_i : in STD_LOGIC;
      spi_dout_o : out STD_LOGIC;
      spi_sck_o : out STD_LOGIC;
      spi_cs_adc_n : out STD_LOGIC;
      spi_cs_dac1_n : out STD_LOGIC;
      spi_cs_dac2_n : out STD_LOGIC;
      spi_cs_dac3_n : out STD_LOGIC;
      spi_cs_dac4_n : out STD_LOGIC;
      si570_scl : inout STD_LOGIC;
      si570_sda : inout STD_LOGIC;
      mezz_one_wire : inout STD_LOGIC;
      si570_oe : out STD_LOGIC;
      gpio_ssr_ch1_o : out STD_LOGIC_VECTOR(6 downto 0);
      gpio_ssr_ch2_o : out STD_LOGIC_VECTOR(6 downto 0);
      gpio_ssr_ch3_o : out STD_LOGIC_VECTOR(6 downto 0);
      gpio_ssr_ch4_o : out STD_LOGIC_VECTOR(6 downto 0);
      gpio_led_trig_o : out STD_LOGIC;
      gpio_led_acq_o : out STD_LOGIC;
      gpio_dac_clr_n_o : out STD_LOGIC;
      acq_cfg_ok_o : out STD_LOGIC
    );
    end component pxie_bd;

begin  -- architecture arch

  adc_gpio_si570_oe_o <= '0';

  -- reset synchronisation
  reset_sync : xpm_cdc_sync_rst
   generic map (
      DEST_SYNC_FF => 10,
      INIT => 0
   ) port map (
      dest_rst => resetn,
      dest_clk => pl_sys_clk,
      src_rst => ps2pl_resetn
   );

  -- PXIe diff clocks/sync pins
  ibufds_pxie_dstara: IBUFDS
  port map (
     O => pxie_dstara,
     I => pxie_dstara_p_i,
     IB => pxie_dstara_n_i
  );

  ibufds_pxie_dstarb: IBUFDS
  port map (
     O => pxie_dstarb,
     I => pxie_dstarb_p_i,
     IB => pxie_dstarb_n_i
  );

  obufds_pxie_dstarc : OBUFDS
   generic map (
      SLEW => "FAST")
   port map (
      O => pxie_dstarc_p_o,
      OB => pxie_dstarc_n_o,
      I => pxie_dstarc
   );

  ibufds_pxie_sync100: IBUFDS
  port map (
     O => pxie_sync100,
     I => pxie_sync100_p_i,
     IB => pxie_sync100_n_i
  );

  ibufds_pxie_clk100: IBUFDS
  port map (
     O => pxie_clk100_bufg,
     I => pxie_clk100_p_i,
     IB => pxie_clk100_n_i
  );

  bufg_pxieclk100 : BUFG
   port map (
      O => pxie_clk100,
      I => pxie_clk100_bufg
   );

  bufgce_pxi10 : BUFGCE
   port map (
      O => open,
      CE => '1',
      I => pxie_clk10_i
   );

  -- PL system clock from SI5341
  ibufds_clksys: IBUFDS
  port map (
     O => sysclk_bufg,
     I => pl_sys_clk_p_i,
     IB => pl_sys_clk_n_i
  );

  bufg_sysclk: BUFG
  port map (
    I => sysclk_bufg,
    O => pl_sys_clk
  );

  -- I2C buffers
  i2c_scl_iobuf: IOBUF
   port map (
      O => i2c_scl_i,
      IO => i2c_scl_io,
      I => i2c_ps0_scl_o,
      T => i2c_ps0_scl_t
   );

  i2c_sda_iobuf: IOBUF
   port map (
      O => i2c_sda_i,
      IO => i2c_sda_io,
      I => i2c_ps0_sda_o,
      T => i2c_ps0_sda_t
   );

  pxie_bd_i: pxie_bd
  port map (
    clk_i => pl_sys_clk,
    ps2pl_resetn_o => ps2pl_resetn,
    resetn_i => resetn,

    ddr4_act_n => ddr4_act_n,
    ddr4_adr => ddr4_adr,
    ddr4_ba => ddr4_ba,
    ddr4_bg => ddr4_bg,
    ddr4_ck_c => ddr4_ck_c,
    ddr4_ck_t => ddr4_ck_t,
    ddr4_cke => ddr4_cke,
    ddr4_cs_n => ddr4_cs_n,
    ddr4_dm_n => ddr4_dm_n,
    ddr4_dq => ddr4_dq,
    ddr4_dqs_c => ddr4_dqs_c,
    ddr4_dqs_t => ddr4_dqs_t,
    ddr4_odt => ddr4_odt,
    ddr4_reset_n => ddr4_reset_n,
    i2c_ps0_scl_i => i2c_scl_i,
    i2c_ps0_scl_o => i2c_ps0_scl_o,
    i2c_ps0_scl_t => i2c_ps0_scl_t,
    i2c_ps0_sda_i => i2c_sda_i,
    i2c_ps0_sda_o => i2c_ps0_sda_o,
    i2c_ps0_sda_t => i2c_ps0_sda_t,
    pl_refddr_clk_n => pl_refddr_clk_n,
    pl_refddr_clk_p => pl_refddr_clk_p,
    
    ps_gpio_i => ps_gpio_i,
    ps_gpio_o => ps_gpio_o,
    
    --wr_clk62m5_o => open,
    --wr_eeprom_scl_io => wr_eeprom_scl_io,
    --wr_eeprom_sda_io => wr_eeprom_sda_io,
    --wr_led_act_o => open,
    --wr_led_link_o => open,
    --wr_pll20dac_csn_o => pll20dac_cs_n_o,
    --wr_pll25dac_csn_o => pll25dac_cs_n_o,
    --wr_plldac_din_o => plldac_din_o,
    --wr_plldac_sclk_o => plldac_sclk_o,
    --wr_pps_led_o => open,
    --wr_pps_o => wr_pps_o,
    ext_trigger_p => adc_ext_trigger_p_i,
    ext_trigger_n => adc_ext_trigger_n_i,
    adc_dco_p => adc_dco_p_i,
    adc_dco_n => adc_dco_n_i,
    adc_fr_p => adc_fr_p_i,
    adc_fr_n => adc_fr_n_i,
    adc_outa_p => adc_outa_p_i,
    adc_outa_n => adc_outa_n_i,
    adc_outb_p => adc_outb_p_i,
    adc_outb_n => adc_outb_n_i,
    spi_din_i => adc_spi_din_i,
    spi_dout_o => adc_spi_dout_o,
    spi_sck_o => adc_spi_sck_o,
    spi_cs_adc_n => adc_spi_cs_adc_n_o,
    spi_cs_dac1_n => adc_spi_cs_dac1_n_o,
    spi_cs_dac2_n => adc_spi_cs_dac2_n_o,
    spi_cs_dac3_n => adc_spi_cs_dac3_n_o,
    spi_cs_dac4_n => adc_spi_cs_dac4_n_o,
    si570_scl => adc_si570_scl_b,
    si570_sda => adc_si570_sda_b,
    si570_oe => open,
    gpio_ssr_ch1_o => adc_gpio_ssr_ch1_o,
    gpio_ssr_ch2_o => adc_gpio_ssr_ch2_o,
    gpio_ssr_ch3_o => adc_gpio_ssr_ch3_o,
    gpio_ssr_ch4_o => adc_gpio_ssr_ch4_o,
    gpio_led_trig_o => adc_gpio_led_trig_o,
    gpio_led_acq_o => adc_gpio_led_acq_o,
    gpio_dac_clr_n_o => adc_gpio_dac_clr_n_o,
    acq_cfg_ok_o => led_o(0),
    mezz_one_wire => adc_one_wire_b
  );

end architecture arch;
