#include <stdlib.h>
#include <sleep.h>
#include "i2c.h"

int i2c_init(struct i2c_dev *i2c) {
	// configure I2C bus 100kHz and enable
	i2c->prer_hi = 0;
	i2c->prer_lo = 199;
	i2c->ctr = I2C_CTRL_EN;

	return 0;
}

int i2c_write(struct i2c_dev *i2c, int8_t dev_addr, int8_t addr, uint8_t *data,
		size_t size) {
	uint32_t i2c_status = -1;

	i2c->tx_rx = dev_addr << 1;
	i2c->cr_sr = I2C_CMD_STA | I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}
	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		return -1;
	}

	i2c->tx_rx = addr;
	i2c->cr_sr = I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}
	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		return -2;
	}

	for (size_t i = 0; i < size; i++) {
		i2c->tx_rx = data[i];
		if (i == size - 1) {
			i2c->cr_sr = I2C_CMD_WR | I2C_CMD_STO;
		} else {
			i2c->cr_sr = I2C_CMD_WR;
		}
		i2c_status = -1;
		while ((i2c_status & I2C_STATUS_TIP) != 0) {
			usleep(1);
			i2c_status = i2c->cr_sr;
		}
	}

	return 0;
}

int i2c_write_byte(struct i2c_dev *i2c, int8_t dev_addr, int8_t addr,
		uint8_t data) {
	uint32_t i2c_status = -1;
	int rtn = 0;

	i2c->tx_rx = dev_addr << 1;
	i2c->cr_sr = I2C_CMD_STA | I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c->cr_sr & I2C_STATUS_TIP) != 0) {
		usleep(1);
	}
	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		rtn = 1;
	}

	i2c->tx_rx = addr;
	i2c->cr_sr = I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}
	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		rtn |= 2;
	}

	i2c->tx_rx = data;
	i2c->cr_sr = I2C_CMD_WR | I2C_CMD_STO;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}

	return rtn;
}

int i2c_read(struct i2c_dev *i2c, int8_t dev_addr, int8_t addr, uint8_t *data,
		size_t size) {
	uint32_t i2c_status = -1;
	int rtn = 0;

	i2c->tx_rx = dev_addr << 1;
	i2c->cr_sr = I2C_CMD_STA | I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}
	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		rtn = 1;
	}

	i2c->tx_rx = addr;
	i2c->cr_sr = I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}

	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		rtn |= 2;;
	}

	i2c->tx_rx = (dev_addr << 1) | 1;
	i2c->cr_sr = I2C_CMD_STA | I2C_CMD_WR;
	i2c_status = -1;
	while ((i2c_status & I2C_STATUS_TIP) != 0) {
		usleep(1);
		i2c_status = i2c->cr_sr;
	}

	if (i2c->cr_sr & I2C_STATUS_ACK) {
		i2c->cr_sr = I2C_CMD_STO;
		rtn |= 4;
	}

	for (size_t i = 0; i < size; i++) {
		if (i == size - 1) {
			i2c->cr_sr = I2C_CMD_RD | I2C_CMD_ACK;
		} else {
			i2c->cr_sr = I2C_CMD_RD;
		}

		i2c_status = -1;
		while ((i2c_status & I2C_STATUS_TIP) != 0) {
			usleep(1);
			i2c_status = i2c->cr_sr;
		}

		data[i] = (uint8_t) i2c->tx_rx;
	}

	i2c->cr_sr = I2C_CMD_STO;

	return rtn;
}
