#include <stdlib.h>
#include <sleep.h>

#include "spi.h"
#include "utils.h"

int fa_spi_init(struct spi_dev *spi) {
	spi->divider = 100;

	return 0;
}

int fa_spi_xfer(struct spi_dev *spi, int cs, int num_bits, uint32_t tx,
		uint32_t *rx) {
	uint32_t regval;
	int err = 0;

	/* Put out value (LSB-aligned) in the T0 register (bits 0..31) */
	spi->tx_rx[0] = tx;

	/* Configure SPI controller */
	regval = FA_SPI_CTRL_ASS | /* Automatic Slave Select*/
	FA_SPI_CTRL_Tx_NEG | /* Change on falling edge */
	num_bits; /* In CHAR_LEN field */
	spi->ctrl = regval;

	/* Set Chip Select */
	spi->ss = (1 << cs);

	/* Start transfer */
	spi->ctrl = regval | FA_SPI_CTRL_GO;

	/* Wait transfer complete */
	err = -1;
	for (int i = 0; i < 1000; ++i) {
		usleep(10);
		if (!(spi->ctrl & FA_SPI_CTRL_BUSY)) {
			err = 0;
			break;
		}
	}

	if (err)
		goto out;

	/* Transfer complete, read data */
	regval = spi->tx_rx[0];
	if (rx)
		*rx = regval;
	out:
	/* Clear Chip Select */
	spi->ss = 0;

	return err;
}

